<?php

namespace Teknisa\FIN\Controllers;

use Teknisa\FIN\Services\RecalculoContabilizacaoTitulosBaixasService;
use Teknisa\FIN\Controllers\TeknisaCrud;
use Teknisa\Libs\Service\CRUD as CrudService;
use Teknisa\Libs\Util\Masks\Masks;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Exception;


class RecalculoContabilizacaoTitulosBaixasController extends TeknisaCrud {
    
    protected $dataSourceName;
    
    public function __construct(CrudService $crudService, $parameterBag, RecalculoContabilizacaoTitulosBaixasService $service, Masks $masks) {
        $this->service = $service;
        $this->masks = $masks;
        parent::__construct($crudService, $parameterBag);
    }
    
    public function findTitulosPagar(DTO\Request\Filter $request, Response $response) {
        $this->dataSourceName = 'recalculo_contas_pagar';
        $conditions = $request->getFilterCriteria()->getConditions();
        
        $result = $this->service->findTitulosPagar($conditions);
        $dataSet = new DataSet($this->dataSourceName, $result);
        $response->addDataSet($dataSet);
    }

    public function findTitulosReceber(DTO\Request\Filter $request, Response $response) {
        $this->dataSourceName = 'recalculo_contas_receber';
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = $this->service->findTitulosReceber($conditions);
        $dataSet = new DataSet($this->dataSourceName, $result);
        $response->addDataSet($dataSet);
    }
    
    public function findBaixasPagar(DTO\Request\Filter $request, Response $response) {
        $this->dataSourceName = 'recalculo_baixas_pagar';
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = $this->service->findBaixasPagar($conditions);
        
        for($i = 0; count($result) > $i; $i++) {
            $result[$i]['CDAGENCIAMASK'] = $this->masks->mask($result[$i]['CDAGENCIA'], str_replace('9', '#', $result[$i]['CDPICTAGEN']));
            $result[$i]['CDCONTCORRMASK'] = $this->masks->mask($result[$i]['CDCONTCORR'], str_replace('9', '#', $result[$i]['CDPICTCONT']));
        }
        
        
        $dataSet = new DataSet($this->dataSourceName, $result);
        $response->addDataSet($dataSet);
    }
    
    public function findBaixasReceber(DTO\Request\Filter $request, Response $response) {
        $this->dataSourceName = 'recalculo_baixas_receber';
        $conditions = $request->getFilterCriteria()->getConditions();
        $result = $this->service->findBaixasReceber($conditions);
        
        for($i = 0; count($result) > $i; $i++) {
            $result[$i]['CDAGENCIAMASK'] = $this->masks->mask($result[$i]['CDAGENCIA'], str_replace('9', '#', $result[$i]['CDPICTAGEN']));
            $result[$i]['CDCONTCORRMASK'] = $this->masks->mask($result[$i]['CDCONTCORR'], str_replace('9', '#', $result[$i]['CDPICTCONT']));
        }
        
        
        $dataSet = new DataSet($this->dataSourceName, $result);
        $response->addDataSet($dataSet);
    }
    
    
    public function validateRowTitulo(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        try {
            $this->service->validateRowTitulo($row);
        } catch (\Exception $e) {
            $variables = array();
            if ($e->getCode() == RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE) {
                $lastResult = $this->service->getLastSelectContopectblResult();
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE;
                $variables['DTINICCTBL'] = substr($lastResult['DTINICCTBL'], 0, -9);
                $variables['DTFINACCTBL'] = substr($lastResult['DTFINACCTBL'], 0, -9);
            }
            else {
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE;
            }
            $message = new Message(null, null, null, $variables);
            $response->addMessage($message);
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function validateRowBaixaPagar(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        try {
            $this->service->validateRowBaixaPagar($row);
        } catch (\Exception $e) {
            $variables = array();
            if ($e->getCode() == RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE) {
                $lastResult = $this->service->getLastSelectContopectblResult();
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE;
                $variables['DTINICCTBL'] = substr($lastResult['DTINICCTBL'], 0, -9);
                $variables['DTFINACCTBL'] = substr($lastResult['DTFINACCTBL'], 0, -9);
            }
            else {
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE;
            }
            $message = new Message(null, null, null, $variables);
            $response->addMessage($message);
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function validateRowBaixaReceber(DTO\Request\Row $request, DTO\Response $response) {
        
        $row = $request->getRow();
        try {
            $this->service->validateRowBaixaReceber($row);
        } catch (\Exception $e) {
            $variables = array();
            if ($e->getCode() == RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE) {
                $lastResult = $this->service->getLastSelectContopectblResult();
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::FORA_DO_PERIODO_CODE;
                $variables['DTINICCTBL'] = substr($lastResult['DTINICCTBL'], 0, -9);
                $variables['DTFINACCTBL'] = substr($lastResult['DTFINACCTBL'], 0, -9);
            }
            else {
                $variables['CODE'] = RecalculoContabilizacaoTitulosBaixasService::LANC_INVALIDO_CODE;
            }
            $message = new Message(null, null, null, $variables);
            $response->addMessage($message);
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function validateMultipleRowsTitulos(DTO\Request\DataSet $request, DTO\Response $response) {
    
        $rows = $request->getDataSet()->getRows();
        
        $responseDataSource = new DataSet('rows_to_be_unchecked', $this->service->getRowsToBeUncheckedTitulos($rows));
        
        $response->addDataSet($responseDataSource);
        
    }
    
    public function validateMultipleRowsBaixasPagar(DTO\Request\DataSet $request, DTO\Response $response) {
    
        $rows = $request->getDataSet()->getRows();
        
        $responseDataSource = new DataSet('rows_to_be_unchecked', $this->service->getRowsToBeUncheckedBaixasPagar($rows));
        
        $response->addDataSet($responseDataSource);
        
    }
    
    public function validateMultipleRowsBaixasReceber(DTO\Request\DataSet $request, DTO\Response $response) {
    
        $rows = $request->getDataSet()->getRows();
        
        $responseDataSource = new DataSet('rows_to_be_unchecked', $this->service->getRowsToBeUncheckedBaixasReceber($rows));
        
        $response->addDataSet($responseDataSource);
        
    }
    
    public function recontabilizarLancamentos(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->recontabilizarLancamentos($rows, $response);
        } catch(\Exception $e) {
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function recontabilizarRecebimentos(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->recontabilizarRecebimentos($rows, $response);
        } catch(\Exception $e) {
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function recontabilizarBaixasPagar(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->recontabilizarBaixasPagar($rows, $response);
        } catch(\Exception $e) {
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
    public function recontabilizarBaixasReceber(DTO\Request\DataSet $request, DTO\Response $response) {
        
        $rows = $request->getDataSet()->getRows();
        try {
            $this->service->recontabilizarBaixasReceber($rows, $response);
        } catch(\Exception $e) {
            $response->setStatus(Response::STATUS_ERROR);
        }
        
    }
    
} 