<?php

namespace Teknisa\FIN\Modules\ContaReceber\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Geral\Entities\Endefili;
use Teknisa\Model\Geral\Entities\Endeclie;
use Teknisa\Model\Geral\Entities\Contcorr;
use Teknisa\Model\Financeiro\Entities\GeralRec;
use Teknisa\Libs\Util\Environment;

/**
 * Class EnvioAutomaticoFactory
 * @package Teknisa\FIN\Modules\ContaReceber\Factory
 */
class EnvioAutomaticoFactory {

    protected $entityManager;
    protected $connectionCustom;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager, $connectionCustom, Environment $environment) {
        $this->connection    = $entityManager->getConnection();
        $this->entityManager = $entityManager;
        $this->connectionCustom = $connectionCustom;
        $this->environment      = $environment;
    }
    
    /**
     * Busca GeralRec
     * @param $nrorg
     * @return GeralRec
     */
    public function buscaGeralRec($nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;

        /** @var GeralRec $geralrec */
        $geralrec = $this->entityManager->getRepository(GeralRec::class)->findOneBy($params);

        return $geralrec;
    }
    
    /**
     * Busca Contcorr
     * @param $cdreducont
     * @return Contcorr
     */
    public function buscaContcorr($cdreducont){
        /** @var $params */
        $params = array();
        $params['cdreducont'] = $cdreducont;

        /** @var Contcorr $contcorr */
        $contcorr = $this->entityManager->getRepository(Contcorr::class)->findOneBy($params);

        return $contcorr;
    }
    
    /**
     * Busca Endefili
     * @param $cdfilial
     * @param $idtpendefili
     * @return Endefili
     */
    public function buscaEndefili($cdfilial, $idtpendefili){
        /** @var $params */
        $params = array();
        
        $params['cdfilial']     = $cdfilial;
        $params['idtpendefili'] = $idtpendefili;

        /** @var Endefili $endefili */
        $endefili = $this->entityManager->getRepository(Endefili::class)->findOneBy($params);
        
        if(!$endefili && $idtpendefili != 'P'){
            $params['idtpendefili'] = 'P';
            
            /** @var Endefili $endefili */
            $endefili = $this->entityManager->getRepository(Endefili::class)->findOneBy($params);
        }

        return $endefili;
    }
    
    /**
     * Busca Endeclie
     * @param $cdcliente
     * @param $idtpendeclie
     * @return Endeclie
     */
    public function buscaEndeclie($cdcliente, $idtpendeclie){
        /** @var $params */
        $params = array();
        
        $params['cdcliente']    = $cdcliente;
        $params['idtpendeclie'] = $idtpendeclie;

        /** @var Endeclie $endeclie */
        $endeclie = $this->entityManager->getRepository(Endeclie::class)->findOneBy($params);
        
        if(!$endeclie && $idtpendeclie != 'P'){
            $params['idtpendeclie'] = 'P';
            
            /** @var Endeclie $endeclie */
            $endeclie = $this->entityManager->getRepository(Endeclie::class)->findOneBy($params);
        }

        return $endeclie;
    }

    /**
     * Busca Títulos para Envio Automático por E-mail
     * @param $qtdiaposenvcar
     * @param $cdtpbairenvbol
     * @return array
     */
    public function buscaEnvioTitulo($qtdiaantenvbol, $cdtpbairenvbol){
        /** @var $params */
        $params = array();

        $params['P_QTDIAANTENVBOL'] = $qtdiaantenvbol;
        $params['P_CDTPBAIRENVBOL'] = $cdtpbairenvbol;

        $sql = " SELECT BR.NRORG, BR.CDEMPRESA, BR.CDFILIAL, BR.NRLANBOLREC, BR.DTVENBOLREC, BR.CDCLIENTE, BR.CDREDUCONT, BR.CDCARTCOBBOL, BR.VRFINALBREC,CC.CDBANCO, CC.CDCEDENTEBAN
                   FROM BOLETRECEBER BR, CONTCORR CC
                  WHERE BR.DTVENBOLREC = (TRUNC(SYSDATE) + :P_QTDIAANTENVBOL)
                    AND INSTR(:P_CDTPBAIRENVBOL, BR.CDTPBAIXAREC) > 0
                    AND BR.IDSTATBREC = 'E'
                    AND BR.CDREDUCONT = CC.CDREDUCONT(+)";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }

    /**
     * Busca Títulos para Envio Automático de Carta de Cobrança por E-mail
     * @param $qtdiaposenvcar
     * @param $cdtpbairenvbol
     * @return array
     */
    public function buscaEnvioCartaCobranca($qtdiaposenvcar, $cdtpbairenvbol){
        /** @var $params */
        $params = array();

        $params['P_QTDIAPOSENVCAR'] = $qtdiaposenvcar;
        $params['P_CDTPBAIRENVBOL'] = $cdtpbairenvbol;

        $sql = " SELECT BR.NRORG, BR.CDEMPRESA, BR.CDFILIAL, BR.NRLANBOLREC, BR.DTVENBOLREC, BR.CDCLIENTE, BR.VRFINALBREC
                   FROM BOLETRECEBER BR
                  WHERE BR.DTVENBOLREC = (TRUNC(SYSDATE) - :P_QTDIAPOSENVCAR)
                    AND INSTR(:P_CDTPBAIRENVBOL, BR.CDTPBAIXAREC) > 0
                    AND BR.IDSTATBREC IN ('A', 'E', 'P')";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        
        return $std->fetchAll();
    }
    
    /**
     * Função Responsável por persistir a conexão com o bando de dados
     * @param $nrorg
     * @param $useVpd
     * @param $newEntityManager
     */
    public function conectar($nrorg, $useVpd, $newEntityManager){
        if($useVpd){
            $this->entityManager = $newEntityManager;
            $this->connection = $newEntityManager->getConnection();
        }
        
        $this->environment->setNrOrg($nrorg);
        $this->environment->setNrOrgTrab($nrorg);
        $this->environment->setCurrentUserId('AUTOMATICO');
    }
}
