<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoBanrisulService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoBanrisulService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 8;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';
    const PRODUTO                                = 2;


    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        
        $digito = $this->getMod10Mod11_B7($nossoNumero);
        return $nossoNumero.$digito;
    }
        
    public function getMod10Mod11_B7($nossoNumero)
    {
        $dv1 = $this->getMod10($nossoNumero);
        $dv2 = $this->getMod11_B7($nossoNumero.$dv1);
        while ($dv2 === 'P') {
            $dv1 = substr(strval(intval($dv1) + 1), -1);
            $dv2 = $this->getMod11_B7($nossoNumero.$dv1);
        }
        return $dv1.$dv2;
    }
    
    public function getMod10($nossoNumero) {
        
        //Calcula o primeiro Dígito
        $sequencia   = strrev($nossoNumero);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $result = ((int)substr($sequencia, $i, 1) * $peso);
            if($result > 9){
                $acumulador += $result - 9;
            }else{
                $acumulador += $result;
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);
        }
        
        $resto = (int)$acumulador % 10;
        $dv1 = ($resto == 0) ? 0 : (10 - $resto);
        return $dv1;
    }
    
    public function getDacAutoconferencia($string) {
        
        $sequencia   = strrev($string);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $result = ((int)substr($sequencia, $i, 1) * $peso);
            $acumulador += $result;
            $peso = $this->incrementaValorPeso($peso, 2, 9);
        }
        $resto = (int)$acumulador % 11;
        $dv1 = ($resto <= 1) ? 0 : (11 - $resto);
        return $dv1;
    }
    
    public function getMod11_B7($stCodigo) {
        $result = '';
        if (!empty($stCodigo)) {
            $loSoma = 0;
            $byBase = 2;
            for ($i = strlen($stCodigo) - 1; $i >= 0; $i--) {
                // Verifica se o caracter é um número
                try {
                    $inCarac = intval($stCodigo[$i]);
                } catch (Exception $e) {
                    $inCarac = 0;
                }
                $loSoma += $inCarac * $byBase;
                // Incrementa a base até chegar a 7
                $byBase++;
                if ($byBase > 7) {
                    $byBase = 2;
                }
            }
            $inResto = ($loSoma % 11);
            switch ($inResto) {
                case 0:
                    $result = '0';
                    break;
                case 1:
                    $result = 'P';
                    break;
                default:
                    $result = strval(11 - $inResto);
                    break;
            }
        }
        return $result;
    }

    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Banrisul.
     * @param $nossoNumero
     * @return string
     */

    public function getLinhaDigitavel($nossoNumero)
    {
		$grupo1 = $this->getIdentificadorBanco().self::MOEDA_REAL;
		$grupo1 .= self::PRODUTO;
        $grupo1 .= '1';
        $grupo1 .= substr($this->getCdAgencia(), 0, 3);
		$grupo1 .= $this->getMod10($grupo1);
		
        $grupo2  = substr($this->getCdAgencia(), 3, 1);
		$grupo2 .= substr($this->getCdcedenteban(), 0, 7);
        $grupo2 .= substr($nossoNumero, 0, 2);
		$grupo2 .= $this->getMod10($grupo2);
        
		$grupo3 = substr($nossoNumero,2, 6);
        $grupo3 .= '40';
        $grupo3 .= substr($this->getCodigoDeBarras($nossoNumero),42,2);
        $grupo3 .=  $this->getMod10($grupo3);

        $grupo4 = substr($this->getCodigoDeBarras($nossoNumero),4,1);
		$grupo5 = $this->getFatorVencimento().$this->getValorNominal();
		
		return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        
        //Esta string será substituída pelo código do dígito verificador
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        $codigoDeBarras .= $this->getFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        $codigoDeBarras .= self::PRODUTO;
        $codigoDeBarras .= '1';
		$codigoDeBarras .= substr($this->getCdAgencia(), 0, 4);
		$codigoDeBarras .= substr($this->getCdcedenteban(), 0, 7);
		$codigoDeBarras .= substr($nossoNumero, 0, 8);
        $codigoDeBarras .= '40';
        $pos20a42 = self::PRODUTO.'1'.substr($this->getCdAgencia(), 0, 4).substr($this->getCdcedenteban(), 0, 7).substr($nossoNumero, 0, 8).'40';
        $codigoDeBarras .= $this->getMod10Mod11_B7($pos20a42);
        $digito         = $this->getDacAutoconferencia(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);
        return $codigoDeBarras;
    }

	private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }

    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

	private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

       	$linhadigitavel = substr($grupo1,0,5).'.'.substr($grupo1, 5).' ';

       	$linhadigitavel .= substr($grupo2,0, 5).'.'.substr($grupo2, 5).' ';

       	$linhadigitavel .= substr($grupo3,0, 5).'.'.substr($grupo3, 5).' ';

       	$linhadigitavel .= $grupo4.' ';

       	$linhadigitavel .= $grupo5;

       	return $linhadigitavel;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
}
