<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoBrasilService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoDaycovalService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 10;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 11;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';


    public function getNossoNumeroComDigito()
    {
        // 4 dígitos do código da Agencia do Título, sem DV
        $agencia = $this->getCdAgencia();
        
        // 3 Dígitos do codigo da carteira
        $carteira = $this->getCarteira();
        
        // 10 dígitos do nosso numero
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        $dv = $this->calculaDVNossoNumero($agencia.$carteira.$nossoNumero);
        return $nossoNumero.$dv;
    }


    
    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Bradesco.
     * @param $nossoNumero
     * @return string
     */
    public function getLinhaDigitavel($nossoNumero)
    {
        // Grupo 1: Composto pelo código do banco, código da moeda as cinco primeiras posições do campo livre do cód de barras, dígito verificador deste campo; BBBMLLLLLD,
        $grupo1  = $this->getIdentificadorBanco().self::MOEDA_REAL.str_pad(substr($this->getCdAgencia(), 0, 4), 3, "0", STR_PAD_LEFT).substr(str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT),0,1);
        $grupo1 .= $this->getVerificadorGruposLinhaDigitavel($grupo1,'grupo1');

        // * Grupo 2: Composto pelas posições 6 a 15 do campo livre do cód de barras e dígito verificador deste campo;
        //Parte da Carteira/Operação/Nosso Numero 
        $grupo2  = substr(str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT),1,2);
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($this->boleto->getNrorg(),$this->boleto->getCdreducont());
        $grupo2 .= str_pad($contcorr->getCdopercontcorr(), 7, "0", STR_PAD_LEFT);
        $grupo2 .= substr($nossoNumero,0,1);
        $grupo2 .= $this->getVerificadorGruposLinhaDigitavel($grupo2,"grupo2");

        //Campo 3: Composto pelas posições 16 a 25 do campo livre do cód de barras e dígito verificador deste campo;
        $grupo3 = substr($nossoNumero,1,10);
        $grupo3 .= $this->getVerificadorGruposLinhaDigitavel($grupo3,'grupo3');

        // Grupo 4: Dígito verificador do código de barras;
        $grupo4 = $this->getDigitoCodigoBarras();

        //Campo 5: Composto pelo fator de vencimento e pelo valor nominal do documento sem edição/dedução (sem ponto e vírgula).
        $grupo5 = $this->getFatorVencimento();
        $grupo5 .= $this->getValorNominal();

        return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        // 3 Posições do Código do banco
        $codigoDeBarras  = $this->getIdentificadorBanco();
        
        // 1 Posição código da moeda
        $codigoDeBarras .= self::MOEDA_REAL;
        
        //DV Codigo de barras
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        
        // 4 Posições do Fator de Vencimento
        $codigoDeBarras .= $this->getFatorVencimento();
        
        //10 Posições do Valor do Título 
        $codigoDeBarras .= $this->getValorNominal();
        
        //25 Posições do Campo Livre
        //4 Posições da Agencia sem o DV
		$codigoDeBarras .= str_pad(substr($this->getCdAgencia(), 0, 4), 4, "0", STR_PAD_LEFT);
		
		//3 Posições da Carteira
		$codigoDeBarras .= substr(str_pad($this->boleto->getCdcartcobbol(), 3, "0", STR_PAD_LEFT),0,3);
		
		//7 Posições  Operação
		$contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($this->boleto->getNrorg(),$this->boleto->getCdreducont());
        $codigoDeBarras .= str_pad($contcorr->getCdopercontcorr(), 7, "0", STR_PAD_LEFT);

		//11 Posições do nosso numero mais o DV
        $codigoDeBarras .= substr($nossoNumero, 0, 11);
 		$digito         = $this->calculaDVCodigoBarras(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);
         
        return $codigoDeBarras;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }


    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 4);
        }

        return $codigoCedente;
    }


    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel, $grupo)
    {
        if($grupo == 'grupo1' || $grupo == 'grupo2'){
            $peso = 2;
        }else{
            $peso = 1;
        }
        $total = 0;

        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;

            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2 );

       }
       
       $resultMultiplicacao = $total * 9;
       
       $digito = $resultMultiplicacao % 10;

       return $digito;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

	private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

       	$linhadigitavel = substr($grupo1,0,5).'.'.substr($grupo1, 5).' ';

       	$linhadigitavel .= substr($grupo2,0, 5).'.'.substr($grupo2, 5).' ';

       	$linhadigitavel .= substr($grupo3,0, 5).'.'.substr($grupo3, 5).' ';

       	$linhadigitavel .= $grupo4.' ';

       	$linhadigitavel .= $grupo5;

       	return $linhadigitavel;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
    
    private function calculaDVNossoNumero($nossoNumero)
    {
        $nsnum = strrev($nossoNumero);
        $peso = 2;
        $total = 0;
		$multiplo = 0;
        for($i = 0; $i<=strlen($nsnum);$i++){
            $result = (int) substr($nsnum,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($nsnum,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2 );
       }
       
       
       $resto = $total % 10;
       $dv = ($resto == 0) ? 0 : (10 - $resto);
       return $dv;
    }
    
    private function calculaDVCodigoBarras($sequencia)
    {
        $peso = 4;
        $acumulador = 0;

        for ($i = 0; $i < strlen($sequencia); $i++) {
            $acumulador += ((int)substr($sequencia, $i, 1) * $peso);
            $peso = $this->incrementaValorPesoBrasil($peso, 2, 9);
        }
        
        $total = $acumulador;
        $resto = $total % 11 ;
        $digito =  11 - $resto;
        if($digito == '0' || $digito == '1'|| $digito > '9'){
            $digito = '1';
        }
        $this->setDigitoCodigoBarras($digito);
        return $digito;
    }
    
}
?>