<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Factories;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Geral\Entities\Paramgeral;
use Teknisa\Libs\Util\DateUtil;

class Bco57000RecalculoOrcamentoCentroCustoFactory{

    public function __construct($entityManager) {
        $this->entityManager = $entityManager;
        $this->connection = $entityManager->getConnection();
    }

    public function getDatasPeriodos($dataInicial, $dataFinal, $nrorg) {
        $params = array(
            'DTINICIAL' => $dataInicial,
            'DTFINAL'   => $dataFinal,
            'NRORG'     => $nrorg
          );
        $sql = "SELECT DISTINCT TO_CHAR(DTMES, 'DD/MM/YYYY') DTMES
                  FROM DATA
                 WHERE (DTDATA between :DTINICIAL and :DTFINAL)
                   AND (NRORG = :NRORG)
                 ORDER BY DTMES";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getValorTotalContas($params) {
        $sql = "SELECT SUM(VRATUAPAG) VRATUAPAG
                  FROM CONTAPAGAR CT,NOTAFISCAL NF
                 WHERE (CT.CDFILIAL    = NF.CDFILIAL)
                   AND (CT.NRLANCTONF  = NF.NRLANCTONF)
                   AND (CT.NRORG       = NF.NRORG)
                   AND (NF.IDSTATUS    IN ('A','V'))
                   AND (CT.CDEMPRESA   = :CDEMPRESA)
                   AND (CDFIMOVFLPAG   = :CDFILIAL)
                   AND (CT.NRORG       = :NRORG)
                   AND (CT.DTMOVFLUXO  BETWEEN :DTMOVFLUXO_INI  AND :DTMOVFLUXO_FIN)";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetch();
    }

    public function getTotalAjusteBoleto($params) {
        $sql = "SELECT VRAJUSTE,
                       TIPOAJUS
                  FROM
                     (SELECT SUM(MF.VRFXAJBOLPAG) VRAJUSTE,
                             'DESCONTO'           TIPOAJUS
                        FROM MOVFXAJBOLP MF,
                             TIPOAJPAGAR TP
                       WHERE (MF.CDEMPRESA   = :CDEMPRESA) AND
                             (MF.CDFILIAL    = :CDFILIAL) AND
                             (MF.NRORG       = :NRORG) AND
                             (MF.NRORG       = TP.NRORG) AND
                             (MF.DTMOVFLUXO BETWEEN :DTMOVFLUXO_INI AND :DTMOVFLUXO_FIN) AND
                             (TP.IDAJUCTPAG = 'D') AND
                             (TP.CDAJUCTPAG = MF.CDAJUCTPAG)
                       UNION ALL
                      SELECT SUM(MF.VRFXAJBOLPAG) VRAJUSTE,
                             'ACRESCIMO'        TIPOAJUS
                        FROM MOVFXAJBOLP MF,
                             TIPOAJPAGAR TP
                       WHERE (MF.CDEMPRESA   = :CDEMPRESA) AND
                             (MF.CDFILIAL    = :CDFILIAL) AND
                             (MF.NRORG       = :NRORG) AND
                             (MF.NRORG       = TP.NRORG) AND
                             (MF.DTMOVFLUXO BETWEEN :DTMOVFLUXO_INI AND :DTMOVFLUXO_FIN) AND
                             (TP.IDAJUCTPAG = 'A') AND
                             (TP.CDAJUCTPAG  = MF.CDAJUCTPAG))";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getParamGeral($nrorg){
        $params = array();
        $params['nrorg'] = $nrorg;
        $paramGeral = $this->entityManager->getRepository(Paramgeral::class)->findOneBy($params);
        return $paramGeral;
    }

    public function getCentcust($cdempresa, $cdfilial, $cdcentcust, $dtorcafina, $nrorg){
        $params = array();
        $params['CDEMPRESA'] = $cdempresa;
        $params['CDFILIAL'] = $cdfilial;
        $params['CDCENTCUST'] = $cdcentcust;
        $params['DTORCAFINA'] = $dtorcafina;
        $params['NRORG'] = $nrorg;
        
        $sql="SELECT CDEMPRESA,
                CDFILIAL,
                CDCENTCUST,
                DTORCAFINA,
                NRORG
				FROM ORCAFINACENTCUST
				WHERE CDEMPRESA  = :CDEMPRESA
				  AND CDFILIAL   = :CDFILIAL
				  AND CDCENTCUST = :CDCENTCUST
				  AND DTORCAFINA = :DTORCAFINA
				  AND NRORG      = :NRORG";

		$params = array("CDEMPRESA"=>$cdempresa,
						"CDFILIAL"=>$cdfilial,
		                "CDCENTCUST"=>$cdcentcust,
		                "DTORCAFINA"=>$dtorcafina,
						"NRORG"=>$nrorg);
		$std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function insereCentcust($vrrealorca, $vrextoleorca, $vrorcafina, $cdempresa, $filial, $cdcentcust, $dataMes, $nrorg){
        $sql = "INSERT INTO ORCAFINACENTCUST
				  (VRREALORCA, VREXTOLEORCA, VRORCAFINA, CDEMPRESA, CDFILIAL, CDCENTCUST, DTORCAFINA, NRORG)
				VALUES
				  (:VRREALORCA, :VREXTOLEORCA, :VRORCAFINA, :CDEMPRESA, :CDFILIAL, :CDCENTCUST, :DTORCAFINA, :NRORG)";

		$params = array("VRREALORCA"=>$vrrealorca,
						"VREXTOLEORCA"=>$vrextoleorca,
						"VRORCAFINA"=>$vrorcafina,
						"CDEMPRESA"=>$cdempresa,
						"CDFILIAL"=>$filial,
		                "CDCENTCUST"=>$cdcentcust,
						"DTORCAFINA"=>$dataMes,
						"NRORG"=>$nrorg);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
    }
    
    public function updateCentcust($vrextoleorca, $vrorcafina, $dataMes, $cdcentcust) {
		$sql="UPDATE ORCAFINACENTCUST SET   
				VREXTOLEORCA = :VREXTOLEORCA,
				VRORCAFINA   = :VRORCAFINA
			WHERE DTORCAFINA = :DTORCAFINA
			  AND CDCENTCUST = :CDCENTCUST";

		$params = array("VREXTOLEORCA"=>$vrextoleorca,
						"VRORCAFINA"=>$vrorcafina,
						"DTORCAFINA"=>$dataMes,
						"CDCENTCUST"=>$cdcentcust);
		$query = $this->connection->prepare($sql);
		$query->execute($params);
	}

    public function getPeriodoCentcust($empresa, $filial, $anoini, $anofim) {
        $params = array(
            'EMPRESA' => $empresa,
            'FILIAL'  => $filial,
            'ANOFIM'  => $anofim,
            'ANOINI'  => $anoini
        );
        $sql = "SELECT CDCENTCUST,
                       CDEMPRESA,
                       DTORCAFINA,
                       VRORCAFINA,
                       VREXTOLEORCA,
                       CDFILIAL
                  FROM ORCAFINACENTCUST
                 WHERE CDEMPRESA  = :EMPRESA AND
                       CDFILIAL   = :FILIAL  AND
                       DTORCAFINA BETWEEN :ANOINI AND :ANOFIM
                 ORDER BY
                       CDEMPRESA,
                       CDFILIAL,
                       CDCENTCUST,
                       DTORCAFINA";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    public function getFilial($cdempresa, $nrorg){
        $params = array(
            'DTINICIAL' => $dataInicial,
            'DTFINAL'   => $dataFinal,
            'NRORG'     => $nrorg
        );
        $sql = "SELECT CDEMPRESA,
                       CDFILIAL,
                       NMFILIAL
                  FROM FILIAL
                 WHERE (CDEMPRESA = :P_EMPRESA)  AND
                       (INSTR(:P_FILIAL, CDFILIAL) <> 0 OR :P_TFILIAL = 'T' )
                 ORDER BY CDFILIAL";
        $std = $this->entityManager->getConnection()->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
}
