<?php

namespace Teknisa\FIN\Modules\MovimentacaoBancaria\Services;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\MovimentacaoBancaria\Factories\Bco57000RecalculoOrcamentoCentroCustoFactory;

class Bco57000RecalculoOrcamentoCentroCustoService{

    protected $dataSourceName, $masks, $service;

    public function __construct($entityManager, $environment, Bco57000RecalculoOrcamentoCentroCustoFactory $bco57000RecalculoOrcamentoCentroCustoFactory) {
        $this->entityManager = $entityManager;
        $this->environment = $environment;
        $this->bco57000RecalculoOrcamentoCentroCustoFactory = $bco57000RecalculoOrcamentoCentroCustoFactory;
        $this->nrorg = $this->environment->getNrOrgTrab();
    }

    /**
     * [saveOrcamentoCentroCusto Realiza o salvamento das cadastramento automático
     * para todas as classificações financeiras por empresa informada.]
     */
    public function saveOrcamentoCentroCusto($orcamentos, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            
            // Busca Parâmetros Gerais
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            
            // Verifica Tipo de Orçamento Centro de Custo
            if ($paramGeral->getIdtporcafina() == 'F')
                 $filial = $row['CDFILIAL'];
            else $filial = $this->environment->getCdfilial();
            
            // Percorre Orçamentos Financeiros
            foreach ($orcamentos as $orcamento) {
                
                // Formata Data de Orçamento Centro de Custo
                $dataMes = '01/'.$orcamento['DTMES'].'/'.$row['DTMES'];
              //  $dataMes = $this->formataData($dataMes);
               
                // Busca Orçamento Centro de Custo
                $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                    $row['CDEMPRESA'], $filial, $row['CDCENTCUST'], $dataMes, $this->nrorg
                );
                
                // Verifica existência Orçamento Centro de Custo
                if (!$centcust){
                    // Insere Orçamento Centro de Custo
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
                        0, $orcamento['VREXTOLEORCA'], $orcamento['VRORCAFINA'], $row['CDEMPRESA'],
                        $filial, $row['CDCENTCUST'], $dataMes, $this->nrorg
                    );
                }else{
                    // Atualiza Orçamento Financeiro
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
                        $orcamento['VREXTOLEORCA'], $orcamento['VRORCAFINA'], $dataMes, $row['CDCENTCUST']
                    );
                   // $centcust = $orcamento['VRORCAFINA'];
                }
                
               // $this->entityManager->persist($centcust);
                $this->entityManager->flush();
            }

            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [CadastramentoAutomaticoDoOrcamento Realiza o cadastramento automático
     * para todas os centros du custos por empresa informada.]
     */
    public function CadastramentoAutomaticoDoOrcamento($row, $origem, $destino) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            $anoInicial = '01/01/'.$row['DTMES'];
            $anoFinal = '31/12/'.$row['DTMES'];

            $tipoCadastramento = null;
            
            if ($paramGeral->getIdtporcafina() == 'F'){
                $tipoCadastramento = 'F';
                $orcamentoOrigem = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getPeriodoCentcust(
                    $origem['CDEMPRESA'],
                    $origem['CDFILIAL'],
                    $anoInicial,
                    $anoFinal
                );

            }else{
                $filialOrigem = $this->environment->getCdfilial();
                $tipoCadastramento = 'E';
                $orcamentoOrigem = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getPeriodoCentcust(
                    $origem['CDEMPRESA'],
                    $filialOrigem,
                    $anoInicial,
                    $anoFinal
                );
            }
            
            foreach ($orcamentoOrigem as $origem) {
                if ($tipoCadastramento == 'E'){
                    $this->efetuaOrcamentoPorEmpresa(
                        $filialOrigem,
                        $origem,
                        $destino,
                        $paramGeral,
                        $row['OPCAOOPERACAO']
                    );
                }else{
                    $this->efetuaOrcamentoPorFilial(
                        $origem,
                        $destino,
                        $row['OPCAOOPERACAO']
                    );
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    /**
     * [efetuaOrcamentoPorFilial Realiza o cadastramento Automático
     * da filial de origem para N filiais de destino]
     *
     * @param  [type] $origem  [Filial do orçamento a ser replicado.
     * Caso a filial não esteja parametrizada na tela orçamento para
     * financeiro, a filial considerada será a filial logada]
     * @param  [type] $destino [Filiais que receberão a replicação]
     * @param  [type] $opcao   [description]
     */
    public function efetuaOrcamentoPorFilial($origem, $destino, $opcao){
        // Unidades para Inclusão/Sobreposição
        $filiais = $destino['CDFILIAL'];
        
        // Percorre Unidades p/ Inclusão/Sobreposição
        foreach ($filiais as $filial) {
            
            // Busca Orçamento Centro de Custo de Destino
            $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                $destino['CDEMPRESA'], $filial, $origem['CDCENTCUST'],
                $this->formataData($origem['DTORCAFINA']), $this->nrorg
            );
            
            // Verifica Inclusão/Sobreposição
            if ($opcao == 'I'){
                if (!$centcustDestino)
                    $this->insereCentcust($origem, $filial, $destino);
            }else{
                if ($centcustDestino){
                    if (($centcustDestino->getCdempresa() != $origem['CDEMPRESA']) || ($centcustDestino->getCdfilial() != $origem['CDFILIAL'])){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) > $centcustDestino->getVrrealorca()){
                            if ($centcustDestino)
                                 $this->updateCentcust($centcustDestino, $origem);
                            else $this->insereCentcust($origem, $filial, $destino);
                        }
                    }
                }else $this->insereCentcust($origem, $filial, $destino);
            }
        }
    }

    /**
     * [efetuaOrcamentoPorEmpresa Realiza o cadastramento Automático
     * da empresa de origem para a empresa de destino]
     */
    public function efetuaOrcamentoPorEmpresa($filial, $origem, $destino, $paramGeral, $opcao){
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
            $destino['CDEMPRESA'],
            $filial,
            $origem['CDCENTCUST'],
            $origem['DTORCAFINA'],
            $this->nrorg
        );
        if ($opcao == 'I'){
            if ($centcustDestino){
                if (($origem['VREXTOLEORCA'] == 0) && ($origem['VRORCAFINA'] == 0)){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $centcustDestino->getVrrealorca()){
                            $this->updateCentcust($centcustDestino, $origem);
                        }
                    }else{
                        $this->updateCentcust($centcustDestino, $origem);
                    }
                }
            }else{
                $this->insereCentcust($origem, $filial, $destino);
            }
        }else{
            if ($centcustDestino){
                if (($destino['CDEMPRESA'] != $origem['CDEMPRESA']) ||
                    ($filial != $origem['CDFILIAL'])){
                    if ($paramGeral->getIdconsorcfin() == 'S'){
                        if (($origem['VRORCAFINA'] + $origem['VREXTOLEORCA']) >= $centcustDestino->getVrrealorca()){
                            $this->updateCentcust($centcustDestino, $origem);
                        }
                    }else{
                        $this->updateCentcust($centcustDestino, $origem);
                    }
                }
            }else{
                $this->insereCentcust($origem, $filial, $destino);
            }
        }
    }

    public function insereCentcust($origem, $cdfilial, $destino){
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
            0,
            $origem['VREXTOLEORCA'],
            $origem['VRORCAFINA'],
            $destino['CDEMPRESA'],
            $cdfilial,
            $origem['CDCENTCUST'],
            $origem['DTORCAFINA'],
            $this->nrorg
        );
        //$this->entityManager->persist($centcustDestino);
        $this->entityManager->flush();
    }

    public function updateCentcust($centcustDestino, $origem){
    /*  $centcustDestino->$origem['VREXTOLEORCA'];
        $centcustDestino->$origem['VRORCAFINA'];
        $this->entityManager->persist($centcustDestino);
        $this->entityManager->flush();
    */
        // update orcamento
        $centcustDestino = $this->bco57000RecalculoOrcamentoCentroCustoFactory->updateCentcust(
            $origem['VREXTOLEORCA'], $origem['VRORCAFINA'], $origem['DTORCAFINA'], $origem['CDCENTCUST']
        );
    }

    public function recalcularOrcamentoCentroCusto($centroCusto, $row) {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            $paramGeral = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getParamGeral($this->nrorg);
            foreach ($centroCusto as $centCusto) {
                $periodos = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getDatasPeriodos(
                    $row['DTORCAFINA_INI'],
                    $row['DTORCAFINA_FIN'],
                    $this->nrorg
                );

                foreach($periodos as $periodo){
                    $valorTotal = 0;
                    list($dia, $mes, $ano) = explode("/", $periodo['DTMES']);
                    $ultimoDia = cal_days_in_month(CAL_GREGORIAN, $mes , $ano);
                    $ultimoDiaMes = $ultimoDia.'/'.$mes.'/'.$ano;
                    $tipoRecalculo = null;
                    $filial = null;
                    
                    if ($paramGeral->getIdtporcafina() == 'F'){
                        $filial = $row['CDFILIAL'];
                        $tipoRecalculo = 'F';
                    }else{
                        $filial = $this->environment->getCdfilial();
                        $tipoRecalculo = 'E';
                    }
                    
                    $valorTotal = $this->getValorTotalTitulos(
                        $row['CDEMPRESA'], $centCusto['CDCENTCUST'],
                        $filial, $periodo['DTMES'], $ultimoDiaMes
                    );
                    $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getCentcust(
                        $row['CDEMPRESA'], $filial, $centCusto['CDCENTCUST'],
                        $this->formataData($periodo['DTMES']), $this->nrorg
                    );
                    if ($centcust){
                        $centcust->setVrrealorca($valorTotal);
                        $this->entityManager->persist($centcust);
                        $this->entityManager->flush();
                    }else{
                        if ($valorTotal != 0){
                            $centcust = $this->bco57000RecalculoOrcamentoCentroCustoFactory->insereCentcust(
                                $valorTotal, 0, 0, $row['CDEMPRESA'], $filial,
                                $centCusto['CDCENTCUST'], $this->formataData($periodo['DTMES']), $this->nrorg
                            );
                            $this->entityManager->persist($centcust);
                            $this->entityManager->flush();
                        }
                    }
                }
            }
            $this->entityManager->flush();
            $connection->commit();
        }catch (\Exception $e){
            $connection->rollback();
            throw $e;
        }
    }

    public function getValorTotalTitulos($cdempresa, $cdcentcust, $cdfilial, $dtmes, $ultimoDiaMes){
        $paramsFilial = array(
            'CDEMPRESA'     => $cdempresa,
            'CDCENTCUST'   => $cdcentcust,
            'NRORG'         => $this->nrorg,
            'CDFILIAL'      => $cdfilial,
            'DTMOVFLUXO_INI' => $dtmes,
            'DTMOVFLUXO_FIN' => $ultimoDiaMes
        );
        $totalTitulo = 0;
        $totalValorAtual = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getValorTotalContas($paramsFilial);
        $ajustesBoleto = $this->bco57000RecalculoOrcamentoCentroCustoFactory->getTotalAjusteBoleto($paramsFilial);
        $totalTitulo = $totalValorAtual['VRATUAPAG'];
        foreach($ajustesBoleto as $ajuste){
            if ($ajuste['VRAJUSTE']){
                if ($ajuste['TIPOAJUS'] == 'DESCONTO'){
                    $totalTitulo = $totalTitulo - $ajuste['VRAJUSTE'];
                }
                if ($ajuste['TIPOAJUS'] == 'ACRESCIMO'){
                    $totalTitulo = $totalTitulo + $ajuste['VRAJUSTE'];
                }
            }
        }
        return $totalTitulo;
    }

    public function formataData($data)
    {
        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDatadeString($data);
        $data = DateUtil::truncateData($data);

        return $data;
    }
}
